package Reader::SLAReport;

#
# Compute sla violations
#
# accepts arguments:
#
#    file="sla report file"
#
# if a file is given, sla thresholds are read from this file,
# otherwise this module expects them on the command line.

use strict;

@Reader::SLAReport::ISA = qw( Reader::Reader );
use cfgfile;
use File::Basename;
use FileHandle;


sub new {
	my ($this) = @_;
	my $class = ref($this) || $this;
	my $self = {};
	bless( $self, $class );
	$self->_init();
	return $self;
}

my %slareport = {};
my %fields = {};


sub _init {
	%slareport = ();
	%fields = ();
}

sub format_field {
	my ( $self, $fieldname ) = @_;
	if ( exists $fields{ $fieldname } ) {
		return $fields{ $fieldname };
	} else {
		BigSister::common::log("warning", "no value for field $fieldname");
	}
	return $fieldname;
}

sub read {
	my ($self) = @_;
	my $cfgfile   = $self->get_argument("file");
	BigSister::common::log( "notice", "reading sla definitions from $cfgfile" );
	
	# read config file
	my $config = new cfgfile( "$cfgfile" );
	unless ( $config->rewind()) {
		BigSister::common::log("err", "failed to read config file $cfgfile");
		return undef;
	} 
	
	while ( $_ = $config->nextline() ) {
		chomp();
		next if ( /^$/ ); # ignore empty lines
		next if ( /^\s*#/ ); # ignoer comments
		if ( /^sla_accept\s*(.*?)\s*$/ ) {
			my $rule = $1;
			if ( $rule =~ /^\s*(.*?)\s*(<|<=|=|>=|>)\s*(.*?)\s*$/ ) {
				my ($service, $criteria, $class, $frame) = split /\./, $1; 
				my ($operator, $threshold_unit) = ($2, $3);
				
				if ( $threshold_unit =~ /\s*(\d+)\s*(%|s|m|h)?\s*/ ) {
					
					$slareport{$service} = undef 
						unless(defined $slareport{$service}); 
					$slareport{$service}{"class"} = $class; 
					$slareport{$service}{"criteria"} = {} 
						unless(defined $slareport{$service}{"criteria"}); 
					$slareport{$service}{"criteria"}{$criteria} = {} 
						unless(defined $slareport{$service}{"criteria"}{$criteria}); 
					$slareport{$service}{"criteria"}{$criteria}{$frame} = {}
						unless(defined $slareport{$service}{"criteria"}{$criteria}{$frame}); 
					$slareport{$service}{"criteria"}{$criteria}{$frame}{"operator"} = $operator;
					$slareport{$service}{"criteria"}{$criteria}{$frame}{"threshold_unit"} = $threshold_unit;
					
					my ($threshold, $unit) = ($1, $2);
					if ( defined $unit ) {
						$slareport{$service}{"criteria"}{$criteria}{$frame}{"unit"} = $unit;
						$threshold = $self->convert_unit(	{
													"from" => "$unit",
													"value" => $threshold
													});
					}	
					$slareport{$service}{"criteria"}{$criteria}{$frame}{"threshold"} = $threshold;
				} else {
					BigSister::common::log( "warning", "failed to parse rule: $rule" );
				}
			} else {
				BigSister::common::log( "warning", "failed to parse rule: $rule" );
			}
		} elsif ( /^sla_format\s*(.*?)\s*=\s*(.*?)\s*$/ ) {
			$fields{$1} = $2;
		} elsif ( /^sla_constant\s*(.*?)\s*=\s*(.*?)\s*$/ ) {
			$fields{$1} = $2;
		} else {
			BigSister::common::log( "info", "unused statement: $_" );
		}
	}	
	
	my $fh = new FileHandle();
	my $stat = $self->get_argument( "stat" );
	foreach $_ ( @$stat ) {
		my $class = $_->{"class"};
		my $frame = $_->{"frame"};
		my $date = $_->{"date"};
		my $statfile = $_->{"file"};
		BigSister::common::log("info", "reading performance data from $statfile");
		open($fh, "< $statfile") || BigSister::common::log( "warning", "failed to open $statfile: $!" );
		while ( $_ = <$fh> ) {
			chomp();
			my $line = $_;
			my ($service, $criteria, $value) = split(",", $_);

			# report only configured values ...
			if ( defined $slareport{$service}
				&& defined $slareport{$service}{"criteria"}{$criteria}
				&& defined $slareport{$service}{"criteria"}{$criteria}{$frame} 
				&& $slareport{$service}{"class"} eq $class ) {
				$slareport{$service}{"criteria"}{$criteria}{$frame}{"statistics"} = {}
					unless(defined $slareport{$service}{"criteria"}{$criteria}{$frame}{"statistics"}); 
				$slareport{$service}{"criteria"}{$criteria}{$frame}{"statistics"}{$date} = $value; 
			}
		}
		close($fh);
	}
}

sub calc_status{
	my ( $self ) = @_;
	foreach my $service (keys %slareport) {
		foreach my $criteria (keys %{$slareport{$service}{"criteria"}} ) {
			foreach my $frame (keys %{$slareport{$service}{"criteria"}{$criteria}} ) {
				my $threshold = $slareport{$service}{"criteria"}{$criteria}{$frame}{"threshold"};
				my $operator = $slareport{$service}{"criteria"}{$criteria}{$frame}{"operator"};
				foreach my $date (keys %{$slareport{$service}{"criteria"}{$criteria}{$frame}{"statistics"}} ) {
					my $stat = _extract_value( $slareport{$service}{"criteria"}{$criteria}{$frame}{"statistics"}{$date}, $operator);
					my $slastatus = "no_data";

					if ( defined $stat ) {
						$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"} = undef
							unless( defined $slareport{$service}{"criteria"}{$criteria}{$frame}{"status"});
						
						# compose expression, replace = by ==
						$operator =~ s/^=/==/;
						my $expression = $stat.$operator.$threshold;
						if( eval( $expression ) ) {
							$slastatus = "sla_ok";
							$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"ok"} = ()
								unless( defined $slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"ok"} );
							push @{$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"ok"}}, $date;
																				
						} else {
							$slastatus = "sla_nok";
							$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"nok"} = ()
								unless( defined $slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"nok"} );
							push @{$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"nok"}}, $date;	
						}
					} else {
						$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"no_status"} = ()
							unless( defined $slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"no_status"} );
						push @{$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"no_status"}}, $date;
					}
					$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"bydate"} = {} 
						unless( defined $slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"bydate"} );
					$slareport{$service}{"criteria"}{$criteria}{$frame}{"status"}{"bydate"}{$date} = $slastatus;	
				}
			}
		}
	}
	return \%slareport;
}

sub convert_unit {
	my ($self, $args) = @_;
	my %factor = (	"m" => 60,
					"h" => 60 * 60,
					"%" => 1,
					"default" => 1
				);
				
	my $factor_from = ( defined $factor{ $args->{"from"} } ) ? $factor{ $args->{"from"} }  : 1;	
	my $factor_to = ( defined $factor{ $args->{"to"} } ) ? $factor{ $args->{"to"} } : 1;
	
	if ( defined $args->{"value"} ) {		
		if ( $args->{"value"} =~ /^\d+(\.\d+)?$/ ) {
			if ( defined $args->{"to"} ) {
				return sprintf("%.2f", $args->{"value"}*$factor_from/$factor_to )." ".$args->{"to"};
			} else {
				return sprintf("%.2f", $args->{"value"}*$factor_from/$factor_to );
			}
		} elsif ( $args->{"value"} =~ /^(.*?)\s*\.{2}\s*(.*?)$/ ) {
			my %arg1 = %{$args};
			$arg1{"value"} = $1;
			my %arg2 = %{$args};
			$arg2{"value"} = $2;
			return $self->convert_unit( \%arg1 )." .. ".$self->convert_unit( \%arg2 );
		} else {
			return $args->{"value"};
		}
	}
	return undef;
}

sub _extract_value {
	my ($value, $operator) = @_;
	if ( $value =~ /^(.*?)\s*\.{2}\s*(.*?)$/ ) {
		my ( $min, $max ) = ($1, $2); 
		if ($operator =~ />/ ) {
			$value = $max;
		} elsif ($operator =~ /</ ) {
			$value =$min;
		} else {
			$value = "N/A";
		}
	}
	return ( $value =~ /^[\d\.]+$/ ) ? $value : undef;
}
